#ifndef CPPOSTEST_H
#define	CPPOSTEST_H

#include <iostream>
#include <vector>

using namespace std;

namespace testing
{
	class Test
	{
	public:
		Test();
		~Test();
		virtual void SetUp();
		virtual void TearDown();
		virtual void RunTest() = 0;
		const char *GetFullName();
		
	protected:
		void SetName();

		const char *CaseName, *TestName;
		char *FullName;
	};

	/* Output interface */
	class Out
	{
	public:
		virtual Out &operator << (const char *s) = 0;
	};
	
	/* Just redirects output to the default stream */
	class EOut : Out
	{
	public:
		Out &operator<<(const char *s)
		{
			cout << s << endl;
			return *this;
		}
	};

	/* Redirects ouput to the default stream and throws an exception to stop the current test */
	class TEOut : Out
	{
	public:
		Out &operator<<(const char *s)
		{
			cout << s << endl;

			throw std::exception();
			return *this;
		}
	};

	/* Stop the propagation of the message. This is used when the test succeeds */
	class DOut : Out
	{
	public:
		Out &operator<<(const char *s)
		{
			return *this;
		}
	};
	
	extern bool failed;
	extern std::vector<Test *> AllTests;
	extern int passCount;
	extern std::vector<Test *> Fails;
	extern EOut ErrorOutput;
	extern TEOut ThrowErrorOutput;
	extern DOut DummyOutput;
	
	void AddTest(Test *tc);
	int doRun();
	void hasFailed();
	Out *eqFail(const char *expected, const char *actual, int line, char *file, bool fail);
	Out *genericFail(const char *x, const char *y, const char *symbol, int line, char *file, bool fail);
	Out *booleanFail(bool actual, bool expected, const char *condition, int line, char *file, bool fail);
	char *strtolower(const char *str1);
	bool doStringTest(const char *str1, const char *str2, bool equality, bool ignoreCase);
	Out *stringFail(const char *expected, const char *actual, bool equality, bool ignoreCase, int line, char *file, bool fail);
	Out *excFail(const char *expr, const char *exc, int line, char *file, bool fail);
	Out *excFail2(const char *expr, int line, char *file, bool fail);
	Out *excFail3(const char *expr, int line, char *file, bool fail);
	Out *pred1Fail(const char *fn, int val1, const char *val1str, int line, const char *file, bool fail);
	Out *pred2Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int line, const char *file, bool fail);
	Out *pred3Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int line, const char *file, bool fail);
	Out *pred4Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int line, const char *file, bool fail);
	Out *pred5Fail(const char *fn, int val1, const char *val1str, int val2, const char *val2str, int val3, const char *val3str, int val4, const char *val4str, int val5, const char *val5str, int line, const char *file, bool fail);
}

/* Call this where you want to start execution of tests */
#define RUN_ALL_TESTS() \
	::testing::doRun();

/* Given TEST_CLASSNAME(TestCase1, Test1) returns TestCase1_Test1_Test */
#define TEST_CLASS_NAME(tc, tn) \
	tc##_##tn##_Test

/* Same but appends _Inst at end */
#define TEST_CLASS_INSTANCE(tc, tn) \
	tc##_##tn##_Test_Inst

/* Creates a class child of ::testing::Test with name tc_tn_Test */
#define TEST(tc, tn) \
	class TEST_CLASS_NAME(tc, tn) : public ::testing::Test \
	{ \
	public: \
		TEST_CLASS_NAME(tc, tn)() \
		{ \
			CaseName = #tc; \
			TestName = #tn; \
			SetName(); \
			AddTest(this); \
		} \
		\
	public: \
		virtual void RunTest(); \
	}; \
	/* Create an instance of this class. In this way the constructor adds this test to the list of all tests */ \
	class TEST_CLASS_NAME(tc, tn) *TEST_CLASS_INSTANCE(tc, tn) = new TEST_CLASS_NAME(tc, tn)(); \
	/* Declaration of the function implemented just above TEST(tc, tn) */ \
	void TEST_CLASS_NAME(tc, tn)::RunTest()	

/* Creates a test based on a test fixture */
#define TEST_F(fixtureName, tn) \
	class TEST_CLASS_NAME(fixtureName, tn) : public fixtureName \
	{ \
	public: \
		TEST_CLASS_NAME(fixtureName, tn)() \
		{ \
			CaseName = #fixtureName; \
			TestName = #tn; \
			SetName(); \
			AddTest(this); \
		} \
		\
	public: \
		virtual void RunTest(); \
	}; \
	/* Create an instance of this class. In this way the constructor adds this test to the list of all tests */ \
	class TEST_CLASS_NAME(fixtureName, tn) *TEST_CLASS_INSTANCE(fixtureName, tn) = new TEST_CLASS_NAME(fixtureName, tn)(); \
	/* Declaration of the function implemented just above TEST(tc, tn) */ \
	void TEST_CLASS_NAME(fixtureName, tn)::RunTest()	

#define GENERIC_TEST(x, y, test, assert)		(*(((x) test (y)) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::genericFail(#x, #y, #test, __LINE__, __FILE__, assert)))
#define BOOLEAN_TEST(cond, expected, assert)		(*(((cond) == expected) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::booleanFail(cond, expected, #cond, __LINE__, __FILE__, assert)))
#define STRING_TEST(exp, act, eq, igncase, assert)	(*(::testing::doStringTest(exp, act, eq, igncase) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::stringFail(exp, act, eq, igncase, __LINE__, __FILE__, assert)))

#define EXPECT_EQ(x, y)		(*(((x) == (y)) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::eqFail(#x, #y, __LINE__, __FILE__, false)))
#define EXPECT_NE(x, y)		GENERIC_TEST(x, y, !=, false)
#define EXPECT_LT(x, y)		GENERIC_TEST(x, y, <, false)
#define EXPECT_LE(x, y)		GENERIC_TEST(x, y, <=, false)
#define EXPECT_GT(x, y)		GENERIC_TEST(x, y, >, false)
#define EXPECT_GE(x, y)		GENERIC_TEST(x, y, >=, false)

#define ASSERT_EQ(x, y)		(*(((x) == (y)) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::eqFail(#x, #y, __LINE__, __FILE__, true)))
#define ASSERT_NE(x, y)		GENERIC_TEST(x, y, !=, true)
#define ASSERT_LT(x, y)		GENERIC_TEST(x, y, <, true)
#define ASSERT_LE(x, y)		GENERIC_TEST(x, y, <=, true)
#define ASSERT_GT(x, y)		GENERIC_TEST(x, y, >, true)
#define ASSERT_GE(x, y)		GENERIC_TEST(x, y, >=, true)

#define EXPECT_TRUE(cond)	BOOLEAN_TEST(cond, true, false)
#define EXPECT_FALSE(cond)	BOOLEAN_TEST(cond, false, false)
#define ASSERT_TRUE(cond)	BOOLEAN_TEST(cond, true, true)
#define ASSERT_FALSE(cond)	BOOLEAN_TEST(cond, false, true)

#define EXPECT_STREQ(exp, act)		STRING_TEST(exp, act, true, false, false)
#define EXPECT_STRNE(exp, act)		STRING_TEST(exp, act, false, false, false)
#define EXPECT_STRCASEEQ(exp, act)	STRING_TEST(exp, act, true, true, false)
#define EXPECT_STRCASENE(exp, act)	STRING_TEST(exp, act, false, true, false)
#define ASSERT_STREQ(exp, act)		STRING_TEST(exp, act, true, false, true)
#define ASSERT_STRNE(exp, act)		STRING_TEST(exp, act, false, false, true)
#define ASSERT_STRCASEEQ(exp, act)	STRING_TEST(exp, act, true, true, true)
#define ASSERT_STRCASENE(exp, act)	STRING_TEST(exp, act, false, true, true)

#define SECCEED()		/* What's the utility of this macro?? */
#define FAIL()			{ ADD_FAILURE_AT(__FILE__, __LINE__); throw std::exception(); }
#define ADD_FAILURE()		ADD_FAILURE_AT(__FILE__, __LINE__)
#define ADD_FAILURE_AT(f, l)	{ cout << f << ":" << l << ": Failure" << endl << "Failed" << endl; }

#define EXPECT_THROW(statement, exc) \
{ \
	bool fired = false; \
	try { statement; } catch (exc) { fired = true; } \
	(*(fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail(#statement, #exc, __LINE__, __FILE__, false))); \
}

#define EXPECT_ANY_THROW(statement) \
{ \
	bool fired = false; \
	try { statement; } catch (...) { fired = true; } \
	(*(fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail2(#statement, __LINE__, __FILE__, false))); \
}

#define EXPECT_NO_THROW(statement) \
{ \
	bool fired = false; \
	try { statement; } catch (...) { fired = true; } \
	(*(!fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail3(#statement, __LINE__, __FILE__, false))); \
}

#define ASSERT_THROW(statement, exc) \
{ \
	bool fired = false; \
	try { statement; } catch (exc) { fired = true; } \
	(*(fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail(#statement, #exc, __LINE__, __FILE__, true))); \
}

#define ASSERT_ANY_THROW(statement) \
{ \
	bool fired = false; \
	try { statement; } catch (...) { fired = true; } \
	(*(fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail2(#statement, __LINE__, __FILE__, true))); \
}

#define ASSERT_NO_THROW(statement) \
{ \
	bool fired = false; \
	try { statement; } catch (...) { fired = true; } \
	(*(!fired ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::excFail3(#statement, __LINE__, __FILE__, true))); \
}

#define EXPECT_PRED1(func, val1) \
	(*(func(val1) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred1Fail(#func, val1, #val1, __LINE__, __FILE__, false)))
#define EXPECT_PRED2(func, val1, val2) \
	(*(func(val1, val2) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred2Fail(#func, val1, #val1, val2, #val2, __LINE__, __FILE__, false)))
#define EXPECT_PRED3(func, val1, val2, val3) \
	(*(func(val1, val2, val3) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred3Fail(#func, val1, #val1, val2, #val2, val3, #val3, __LINE__, __FILE__, false)))
#define EXPECT_PRED4(func, val1, val2, val3, val4) \
	(*(func(val1, val2, val3, val4) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred4Fail(#func, val1, #val1, val2, #val2, val3, #val3, val4, #val4, __LINE__, __FILE__, false)))
#define EXPECT_PRED5(func, val1, val2, val3, val4, val5) \
	(*(func(val1, val2, val3, val4, val5) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred5Fail(#func, val1, #val1, val2, #val2, val3, #val3, val4, #val4, val5, #val5, __LINE__, __FILE__, false)))

#define ASSERT_PRED1(func, val1) \
	(*(func(val1) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred1Fail(#func, val1, #val1, __LINE__, __FILE__, true)))
#define ASSERT_PRED2(func, val1, val2) \
	(*(func(val1, val2) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred2Fail(#func, val1, #val1, val2, #val2, __LINE__, __FILE__, true)))
#define ASSERT_PRED3(func, val1, val2, val3) \
	(*(func(val1, val2, val3) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred3Fail(#func, val1, #val1, val2, #val2, val3, #val3, __LINE__, __FILE__, true)))
#define ASSERT_PRED4(func, val1, val2, val3, val4) \
	(*(func(val1, val2, val3, val4) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred4Fail(#func, val1, #val1, val2, #val2, val3, #val3, val4, #val4, __LINE__, __FILE__, true)))
#define ASSERT_PRED5(func, val1, val2, val3, val4, val5) \
	(*(func(val1, val2, val3, val4, val5) ? (::testing::Out *) (&::testing::DummyOutput) : (::testing::Out *) ::testing::pred5Fail(#func, val1, #val1, val2, #val2, val3, #val3, val4, #val4, val5, #val5, __LINE__, __FILE__, true)))

#endif
